local L = LibStub("AceLocale-3.0"):GetLocale("GuildLotteryRaffle")
if not L then return end

local Variables = {
	{ false, false, false, },		-- [1]	[WINNERS EXIST]			Becomes the name(s) of the First, Second, and/or Third Raffle Prizes when announcing them.
	{ false, false, false, },		-- [2]	[PRIZE EXISTS]			Becomes True if a Raffle Prize exists for First, Second, Third.
	{ false, false, false, },		-- [3]	[WINNERS ANNOUNCED]		First / Second / Third Raffle Place. Becomes True as the mod announces the associated winners.
	{ "", "", "", },				-- [4]	[RAFFLE WINNERS]		Becomes the Name of the First / Second / Third Place Raffle Winners.
	{ "NA", "NA", "NA", },			-- [5]	[RAFFLE TICKETS]		The First / Second / Third Place Winning Ticket number.
	"NA",							-- [6]	[PREVIOUS TICKET]		Used to store the Lottery Winning Ticket number to be placed in the Previous Event data.
	{ false, false, },				-- [7]	[DRAW COMPLETE]			Becomes True once the Lottery / Raffle Event Draw process is complete.
	false,							-- [8]	[BEGIN ANNOUNCE]		Once the mod starts announcing Raffle Winners, this becomes True and is false all other times.
	false,							-- [9]	[ABORT PRINTED]			If the user decides to Abort a Draw, this will become True once the Abort Message is printed; preventing multiple messages from appearing.
	false,							-- [10]	[LOTTERY WINNER]		Becomes True once a Winner has been found for the Lottery
	false,							-- [11]	[WINNER FOUND]			If a Lottery Winner ( FIRST ONLY ) is found, becomes True; telling the mod to clear certain Previous Event data to prevent the next Lottery's Starting Amount from being incorrect.
	false,							-- [12]	[WIN TYPE]				If a Lottery Winner ( FIRST ONLY ) is found, becomes True. Otherwise it stays False; telling the mod when reporting Previous Event data that the winning Jackpot was for Second Place or not at all (if no name found for the Winning Ticket)
	{ 								-- [13]	[JACKPOT LEFT & NUM TICKETS] 
		0,							--			(1) Becomes the Remaining Jackpot if no one wins First Place in a Lottery.
		0, 							--			(2) Becomes the number of Tickets generated when running Advanced Lottery / Raffle Draws.
	},								--
	false,							-- [14]	[ADVANCED]				Becomes True if the user is running the Advanced Draw for their Event.
	"",								-- [15]	[ADV. PREV. TICKET]		When searching for a Winner in Advanced Draws, if a winner isn't found this variable prevents the mod from picking the same number.
	false,							-- [16]	[RAFFLE BUFFER]			Acts as a buffer while the mod completes Advanced Raffle Draws.
	0,								-- [17]	[GENERATED NUM]			Becomes the number of Tickets the mod is required to generate during Advanced Draws.
	false,							-- [18]	[DEBUG PRINTED]			Used for Debug Tracing.
}

local raffleTicketNumbersTable = {}
local raffleWinners = {}
local locale = GetLocale()
local FullPlayerName = (GetUnitName("PLAYER", false) .. "-" .. string.gsub(string.gsub(GetRealmName(), "-", ""), "%s+", ""))
local RandomizationTable = { ["Names"] = {}, ["Numbers"] = {}, ["Totals"] = {}, }
local EventTickets = {}

local profile = {
	buttons = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		tileSize = 4,
		edgeSize = 12,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	},
	colors = {
		buttons = {
			bordercolor = {1, 0.87, 0, 1},
			bgcolor = {0, 0, 0, 0.5},
		},
	},
}

local function FormatNumber(n)
	if n ~= nil then
		if n < 10 then
			n = "0" .. tostring(n)
		end
		return n
	else return "NA" end
end

local function GetTimeStamp()
	local t = GetServerTime()
	if t ~= nil then
		local d = date("*t", t)
		d.month = FormatNumber(d.month)
		d.day = FormatNumber(d.day)
		d.year = FormatNumber(d.year)
		d.hour = FormatNumber(d.hour)
		d.min = FormatNumber(d.min)
		d.sec = FormatNumber(d.sec)
		local s = d.month .. "/" .. d.day .. "/" .. d.year .. " - " .. d.hour .. ":" .. d.min .. "." .. d.sec
		return s
	else
		return "NA"
	end
end

local function GenerateTicket(firstTicket, lastTicket, state)
	local fT = tonumber(firstTicket)
	local lT = tonumber(lastTicket)
	local ticketNumber = tostring(math.random(fT, lT))
	local ta = {}
	local breakOut = false
	local doBreak = false
	local taCount = 0
	local count = lT * 20
	if glrCharacter[state].TicketPool ~= nil then
		for t, v in pairs(glrCharacter[state].TicketPool) do
			table.insert(ta, v)
			taCount = taCount + 1
		end
	end
	for i = 1, count do
		for j = 1, #ta do
			if ta[j] == ticketNumber then
				ticketNumber = tostring(math.random(fT, lT))
			end
		end
		for v = 1, #ta do
			if ta[v] ~= ticketNumber then
				doBreak = true
			elseif ta[v] == ticketNumber then
				doBreak = false
				ticketNumber = tostring(math.random(fT, lT))
			end
			if doBreak then
				break
			end
		end
		if taCount == 0 then
			doBreak = true
		end
		if doBreak then
			breakOut = true
			break
		end
	end
	if breakOut then
		return ticketNumber
	end
end

local function CommaValue(amount)
	local formatted = amount
	while true do
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
		if (k==0) then
			break
		end
	end
	return formatted
end

local function GetValue(value, EventType, NumberType, PlaceType)
	local total = tostring(value)
	local event = EventType
	local NumberValue = NumberType
	local place = PlaceType
	local round = glrCharacter.Data.Settings.RoundJackpot
	if string.find(total, "%.") == nil then
		total = total .. ".0000"
	end
	if string.sub(total, string.find(total, "%.") + 1, string.find(total, "%.") + 1) == "" then
		total = tostring(total) .. "00"
		if string.sub(total, string.find(total, "%.") + 3, string.find(total, "%.") + 3) == "" then
			total = total .. "00"
		elseif string.sub(total, string.find(total, "%.") + 4, string.find(total, "%.") + 4) == "" then
			total = total .. "0"
		end
	elseif string.sub(total, string.find(total, "%.") + 2, string.find(total, "%.") + 2) == "" then
		total = tostring(total) .. "0"
		if string.sub(total, string.find(total, "%.") + 3, string.find(total, "%.") + 3) == "" then
			total = total .. "00"
		elseif string.sub(total, string.find(total, "%.") + 4, string.find(total, "%.") + 4) == "" then
			total = total .. "0"
		end
	elseif string.sub(total, string.find(total, "%.") + 3, string.find(total, "%.") + 3) == "" then
		total = tostring(total) .. "00"
	elseif string.sub(total, string.find(total, "%.") + 4, string.find(total, "%.") + 4) == "" then
		total = tostring(total) .. "0"
	end
	
	local Gold = string.sub(total, 1, string.find(total, "%.") - 1)
	local Silver = string.sub(total, string.find(total, "%.") + 1, string.find(total, "%.") + 2)
	local Copper = string.sub(total, string.find(total, "%.") + 3, string.find(total, "%.") + 4)
	
	if event == "Lottery" and NumberValue == "Jackpot" and round then
		Gold = tonumber(Gold)
		Silver = tonumber(Silver)
		Copper = tonumber(Copper)
		if Copper >= 50 then
			Copper = "00"
			Silver = Silver + 1
			if Silver >= 50 then
				Silver = "00"
				Gold = tostring(Gold + 1)
			else
				Silver = "00"
			end
		elseif Silver >= 50 then
			Copper = "00"
			Silver = "00"
			Gold = tostring(Gold + 1)
		else
			Copper = "00"
			Silver = "00"
		end
		if place then
			local startingGold = tonumber(glrCharacter.Event.Lottery.StartingGold)
			local tickets = glrCharacter.Event.Lottery.TicketsSold
			local price = tonumber(glrCharacter.Event.Lottery.TicketPrice)
			local jackpotGold = startingGold + ( tickets * price )
			local TotalGold = GetValue(jackpotGold, "Lottery", "Jackpot", false)
			local check, _ = string.gsub(string.gsub(TotalGold, ",", ""), L["Value_Gold_Short"], "")
			if tonumber(string.sub(check, string.len(check), string.len(check))) >= 5 and tonumber(check) >= 12 then
				Gold = tostring(Gold + 1)
			else
				Gold = tostring(Gold)
			end
		else
			Gold = tostring(Gold)
		end
	end
	
	if tonumber(Copper) == 0 then
		if tonumber(Silver) == 0 then
			if tonumber(Gold) > 0 then
				Gold = CommaValue(Gold)
				total = Gold .. L["Value_Gold_Short"]
			else
				total = Gold .. L["Value_Gold_Short"] .. " " .. Silver .. L["Value_Silver_Short"] .. " " .. Copper .. L["Value_Copper_Short"]
			end
		else
			if tonumber(Gold) == 0 then
				total = Silver .. L["Value_Silver_Short"]
			else
				Gold = CommaValue(Gold)
				total = Gold .. L["Value_Gold_Short"] .. " " .. Silver .. L["Value_Silver_Short"]
			end
		end
	else
		if tonumber(Silver) == 0 then
			if tonumber(Gold) == 0 then
				total = Copper .. L["Value_Copper_Short"]
			else
				Gold = CommaValue(Gold)
				total = Gold .. L["Value_Gold_Short"] .. " " .. Copper .. L["Value_Copper_Short"]
			end
		else
			if tonumber(Gold) == 0 then
				total = Silver .. L["Value_Silver_Short"] .. " " .. Copper .. L["Value_Copper_Short"]
			else
				Gold = CommaValue(Gold)
				total = Gold .. L["Value_Gold_Short"] .. " " .. Silver .. L["Value_Silver_Short"] .. " " .. Copper .. L["Value_Copper_Short"]
			end
		end
	end
	return total
end

local function FindNumberRange(name)
	local low = nil
	local high = nil
	for i = 1, #RandomizationTable.Names[name] do
		local check = tonumber(RandomizationTable.Names[name][i])
		if low == nil then low = check end
		if high == nil then high = check end
		if i > 1 then
			if low > check then
				if high <= low then
					high = low
				end
				low = check
			elseif high < check then
				high = check
			end
		end
	end
	return low, high
end

local function PrepareNextLottery()
	
	local text = date()
	local dateTime = string.sub(text, 1, string.find(text, " ") - 1)
	local EventStatus = false
	local transfer = { ["Winner"] = Variables[6] }
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - PrepareNextLottery() - Winning Ticket: " .. Variables[6] .. " - First Place? : " .. string.upper(tostring(Variables[12])) )
	
	if glrCharacter.Data.Settings.CurrentlyActiveRaffle then --Checks if a Raffle is currently Active before confirming that both Lottery/Raffle's are inactive, allowing you to change Multi-Guild status on your character (assuming no other events are active on other characters)
		EventStatus = true
	end
	
	GLR_UI.GLR_MainFrame.GLR_AbortRollProcessButton:Hide()
	
	if not Variables[11] then
		glrCharacter.Data.Settings.PreviousLottery.TicketsSold = glrCharacter.Event.Lottery.TicketsSold
		glrCharacter.Data.Settings.PreviousLottery.TicketPrice = tonumber(glrCharacter.Event.Lottery.TicketPrice)
		glrCharacter.Data.Settings.PreviousLottery.Jackpot = tonumber(Variables[13][1])
	elseif Variables[11] then
		glrCharacter.Data.Settings.PreviousLottery.TicketsSold = 0
		glrCharacter.Data.Settings.PreviousLottery.TicketPrice = 0
		glrCharacter.Data.Settings.PreviousLottery.Jackpot = 0
	end
	
	glrCharacter.Data.Defaults.LotteryDate = dateTime
	
	local startingGold = tostring(glrCharacter.Data.Settings.PreviousLottery.Jackpot)
	local jackpotGold = startingGold
	
	GLR_UI.GLR_LotteryInfo.GLR_LotteryDateEditBox:SetText("")
	GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetText(L["No Lottery Active"])
	GLR_UI.GLR_LotteryInfo.GLR_LotteryNameEditBox:SetCursorPosition(0)
	GLR_UI.GLR_LotteryInfo.GLR_StartingGoldEditBox:SetText(startingGold)
	GLR_UI.GLR_LotteryInfo.GLR_JackpotEditBox:SetText(jackpotGold)
	GLR_UI.GLR_LotteryInfo.GLR_MaxTicketsEditBox:SetText(glrCharacter.Data.Defaults.LotteryMaxTickets)
	GLR_UI.GLR_LotteryInfo.GLR_TicketPriceEditBox:SetText(glrCharacter.Data.Defaults.TicketPrice)
	GLR_UI.GLR_LotteryInfo.GLR_SecondPlaceEditBox:SetText(glrCharacter.Data.Defaults.SecondCut)
	GLR_UI.GLR_LotteryInfo.GLR_GuildAmountEditBox:SetText(glrCharacter.Data.Defaults.GuildCut)
	GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText("0")
	GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText("0")
	
	for t, v in pairs(glrCharacter.Lottery) do
		if transfer[t] == nil then
			transfer[t] = {}
		end
		for key, val in pairs(glrCharacter.Lottery[t]) do
			if transfer[t][key] == nil then
				transfer[t][key] = {}
			end
			transfer[t][key] = glrCharacter.Lottery[t][key]
		end
	end
	
	glrCharacter.PreviousEvent.Lottery.Data = transfer
	glrCharacter.PreviousEvent.Lottery.Settings = glrCharacter.Event.Lottery
	glrCharacter.PreviousEvent.Lottery.Settings.RoundJackpot = glrCharacter.Data.Settings.RoundJackpot
	glrCharacter.PreviousEvent.Lottery.Settings.WonType = Variables[12]
	glrCharacter.PreviousEvent.Lottery.Settings.NoGuildCut = glrCharacter.Data.Settings.NoGuildCut
	glrCharacter.PreviousEvent.Lottery.Available = true
	
	glrCharacter.Event.Lottery = nil
	glrCharacter.Event.Lottery = {}
	
	glrCharacter.Lottery.TicketPool = nil
	glrCharacter.Lottery.TicketPool = {}
	
	glrCharacter.Lottery.Entries.TicketRange = nil
	glrCharacter.Lottery.Entries.TicketRange = {}
	
	glrCharacter.Lottery.Entries.TicketNumbers = nil
	glrCharacter.Lottery.Entries.TicketNumbers = {}
	
	glrCharacter.Lottery.Entries.Tickets = nil
	glrCharacter.Lottery.Entries.Tickets = {}
	
	glrCharacter.Lottery.Entries.Names = nil
	glrCharacter.Lottery.Entries.Names = {}
	
	glrCharacter.Lottery.MessageStatus = nil
	glrCharacter.Lottery.MessageStatus = {}
	
	glrTemp.Lottery = nil
	glrTemp.Lottery = {}
	
	glrTempStatus = nil
	glrTempStatus = {}
	
	if glrHistory.ActiveEvents == nil then
		glrHistory.ActiveEvents = {}
	end
	
	if glrHistory.ActiveEvents[FullPlayerName] == nil then
		glrHistory.ActiveEvents[FullPlayerName] = EventStatus
	else
		glrHistory.ActiveEvents[FullPlayerName] = EventStatus
	end
	
	if GLR_UI.GLR_MainFrame:IsVisible() then
		GLR:UpdatePlayersAndTotalTickets()
		GLR:UpdateNumberOfUnusedTickets()
	end
	
	glrCharacter.Data.Settings.CurrentlyActiveLottery = false
	GLR_Global_Variables[2] = true
	C_Timer.After(5, function(self) GLR_U:UpdateInfo() end)
	
	glrLotteryDrawInProgress = false
	
	GLR_UI.GLR_MainFrame.GLR_ConfirmAbortRollProcessBorder:Hide()
	
	Variables[18] = false
	
	if not Variables[14] then
		SendChatMessage(L["Lottery Draw Complete"], "GUILD")
	else
		SendChatMessage(L["Advanced Lottery Draw Complete"], "GUILD")
	end
	
end

local function PrepareNextRaffle()
	
	local dateTime = string.sub(date(), 1, string.find(date(), " ") - 1)
	local EventStatus = false
	local transfer = { ["FirstWinner"] = Variables[5][1], ["SecondWinner"] = Variables[5][2], ["ThirdWinner"] = Variables[5][3] }
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - PrepareNextRaffle() - Winning Ticket(s); FIRST [" .. Variables[5][1] .. "]; SECOND [" .. Variables[5][2] .. "]; THIRD [" .. Variables[5][3] .. "]" )
	
	if glrCharacter.Data.Settings.CurrentlyActiveLottery then --Checks if a Raffle is currently Active before confirming that both Lottery/Raffle's are inactive, allowing you to change Multi-Guild status on your character (assuming no other events are active on other characters)
		EventStatus = true
	end
	
	GLR_UI.GLR_MainFrame.GLR_AbortRollProcessButton:Hide()
	
	glrCharacter.Data.Defaults.LotteryDate = dateTime
	
	GLR_UI.GLR_RaffleInfo.GLR_RaffleDateEditBox:SetText("")
	GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetText(L["No Raffle Active"])
	GLR_UI.GLR_RaffleInfo.GLR_RaffleNameEditBox:SetCursorPosition(0)
	GLR_UI.GLR_RaffleInfo.GLR_RaffleMaxTicketsEditBox:SetText(glrCharacter.Data.Defaults.RaffleMaxTickets)
	GLR_UI.GLR_RaffleInfo.GLR_RaffleTicketPriceEditBox:SetText(glrCharacter.Data.Defaults.TicketPrice)
	GLR_UI.GLR_RaffleInfo.GLR_RaffleFirstPlacePrizeEditBox:SetText("")
	GLR_UI.GLR_RaffleInfo.GLR_RaffleSecondPlacePrizeEditBox:SetText("")
	GLR_UI.GLR_RaffleInfo.GLR_RaffleThirdPlacePrizeEditBox:SetText("")
	GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderLow:SetText("0")
	GLR_UI.GLR_MainFrame.GLR_TicketNumberRangeBorderHigh:SetText("0")
	
	for t, v in pairs(glrCharacter.Raffle) do
		if transfer[t] == nil then
			transfer[t] = {}
		end
		for key, val in pairs(glrCharacter.Raffle[t]) do
			if transfer[t][key] == nil then
				transfer[t][key] = {}
			end
			transfer[t][key] = glrCharacter.Raffle[t][key]
		end
	end
	
	glrCharacter.PreviousEvent.Raffle.Data = transfer
	glrCharacter.PreviousEvent.Raffle.Settings = glrCharacter.Event.Raffle
	glrCharacter.PreviousEvent.Raffle.Available = true
	
	glrCharacter.Data.Raffle.FirstPrizeData = nil
	glrCharacter.Data.Raffle.FirstPrizeData = {}
	glrCharacter.Data.Raffle.FirstPrizeData.ItemLink = {}
	glrCharacter.Data.Raffle.FirstPrizeData.ItemName = {}
	
	glrCharacter.Data.Raffle.SecondPrizeData = nil
	glrCharacter.Data.Raffle.SecondPrizeData = {}
	glrCharacter.Data.Raffle.SecondPrizeData.ItemLink = {}
	glrCharacter.Data.Raffle.SecondPrizeData.ItemName = {}
	
	glrCharacter.Data.Raffle.ThirdPrizeData = nil
	glrCharacter.Data.Raffle.ThirdPrizeData = {}
	glrCharacter.Data.Raffle.ThirdPrizeData.ItemName = {}
	glrCharacter.Data.Raffle.ThirdPrizeData.ItemLink = {}
	
	glrCharacter.Event.Raffle = nil
	glrCharacter.Event.Raffle = {}
	
	glrCharacter.Raffle.TicketPool = nil
	glrCharacter.Raffle.TicketPool = {}
	
	glrCharacter.Raffle.Entries.TicketRange = nil
	glrCharacter.Raffle.Entries.TicketRange = {}
	
	glrCharacter.Raffle.Entries.TicketNumbers = nil
	glrCharacter.Raffle.Entries.TicketNumbers = {}
	
	glrCharacter.Raffle.Entries.Tickets = nil
	glrCharacter.Raffle.Entries.Tickets = {}
	
	glrCharacter.Raffle.Entries.Names = nil
	glrCharacter.Raffle.Entries.Names = {}
	
	glrCharacter.Raffle.MessageStatus = nil
	glrCharacter.Raffle.MessageStatus = {}
	
	glrTemp.Raffle = nil
	glrTemp.Raffle = {}
	
	glrTempStatus = nil
	glrTempStatus = {}
	
	if glrHistory.ActiveEvents == nil then
		glrHistory.ActiveEvents = {}
	end
	
	if glrHistory.ActiveEvents[FullPlayerName] == nil then
		glrHistory.ActiveEvents[FullPlayerName] = EventStatus
	else
		glrHistory.ActiveEvents[FullPlayerName] = EventStatus
	end
	
	if GLR_UI.GLR_MainFrame:IsVisible() then
		GLR:UpdatePlayersAndTotalTickets()
		GLR:UpdateNumberOfUnusedTickets()
	end
	
	glrCharacter.Data.Settings.CurrentlyActiveRaffle = false
	GLR_Global_Variables[2] = true
	C_Timer.After(5, function(self) GLR_U:UpdateInfo() end)
	
	glrRaffleDrawInProgress = false
	
	GLR_UI.GLR_MainFrame.GLR_ConfirmAbortRollProcessBorder:Hide()
	
	if not Variables[14] then
		C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Complete"], "GUILD") end)
	else
		C_Timer.After(1, function(self) SendChatMessage(L["Advanced Raffle Draw Complete"], "GUILD") end)
	end
	
end

---------------------------------------------
-------------- STARTS THE RAFFLE ------------
---------------------------------------------
--------- STARTS RAFFLE, EVEN BEFORE --------
------------ THE SET RAFFLE DATE ------------
---------------------------------------------

local function AnnounceRaffleWinners()
	Variables[8] = true
	if not glrCharacter.Data.Settings.InverseAnnounce then
		if not Variables[3][1] then
			C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce First Ticket"] .. Variables[5][1], "GUILD") end)
			C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 First Winner Found"] .. Variables[4][1] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][1], "GUILD") end)
			C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(7, AnnounceRaffleWinners)
			Variables[3][1] = true
			return
		elseif not Variables[3][2] then
			if glrCharacter.Event.Raffle.SecondPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Second Ticket"] .. Variables[5][2], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Second Winner Found"] .. Variables[4][2] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][2], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			end
			C_Timer.After(7, AnnounceRaffleWinners)
			Variables[3][2] = true
			return
		elseif not Variables[3][3] then
			if glrCharacter.Event.Raffle.ThirdPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Third Ticket"] .. Variables[5][3], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Third Winner Found"] .. Variables[4][3] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][3], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			end
			Variables[7][2] = true
			Variables[3][3] = true
		end
	else
		if not Variables[3][3] then
			if glrCharacter.Event.Raffle.ThirdPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Third Ticket"] .. Variables[5][3], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Third Winner Found"] .. Variables[4][3] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][3], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			end
			C_Timer.After(7, AnnounceRaffleWinners)
			Variables[3][3] = true
			return
		elseif not Variables[3][2] then
			if glrCharacter.Event.Raffle.SecondPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Second Ticket"] .. Variables[5][2], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Second Winner Found"] .. Variables[4][2] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][2], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			end
			C_Timer.After(7, AnnounceRaffleWinners)
			Variables[3][2] = true
			return
		elseif not Variables[3][1] then
			C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce First Ticket"] .. Variables[5][1], "GUILD") end)
			C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 First Winner Found"] .. Variables[4][1] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][1], "GUILD") end)
			C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[3][1] = true
			Variables[7][2] = true
		end
	end
end

local function RaffleDrawThirdPlace()
	
	local firstTicket = glrCharacter.Event.Raffle.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Raffle.LastTicketBought - 1
	local thirdFound = false
	local prepareNext = false
	local doStep = glrCharacter.Data.Settings.AllowMultipleRaffleWinners
	local continue = true
	local count = 0
	
	for i = 1, #glrCharacter.Raffle.Entries.Names do
		count = count + 1
		if count == 4 then --Don't want to use up too much processing power if their are a lot of entries.
			break
		end
	end
	
	if glrCharacter.Event.Raffle.ThirdPlace ~= false then
		if Variables[2][1] and Variables[2][2] then
			for i = 1, #raffleTicketNumbersTable do
				if raffleTicketNumbersTable[i] == Variables[5][3] then
					for t, v in pairs(glrCharacter.Raffle.Entries.Names) do
						for j = 1, #glrCharacter.Raffle.Entries.TicketNumbers[v] do
							
							local tn = glrCharacter.Raffle.Entries.TicketNumbers[v][j]
							
							if tn == Variables[5][3] then
								
								if count >= 3 and doStep then
									for k = 1, #raffleWinners do
										if raffleWinners[k] == v then
											continue = false
										end
									end
								end
								
								if not GLR_Global_Variables[3] and continue then
									Variables[4][3] = v
									thirdFound = true
									prepareNext = true
									Variables[2][3] = true
									break
								elseif GLR_Global_Variables[3] and not Variables[9] then
									C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
									glrRaffleDrawInProgress = false
									Variables[9] = true
								end
								
							end
						end
						
						if thirdFound or not continue then
							break
						end
						
					end
				end
				
				if thirdFound or not continue then
					break
				end
				
			end
		end
		
	elseif glrCharacter.Event.Raffle.thirdPlace == false then
		thirdFound = true
		prepareNext = true
		Variables[2][3] = true
	end
	
	if not thirdFound then
		if not GLR_Global_Variables[3] then
			if Variables[2][1] and Variables[2][2] then --Prevents redrawing a different ticket number while searching for a first and second place winner.
				Variables[5][3] = GenerateTicket(firstTicket, lastTicket, "Raffle")
			end
			C_Timer.After(1, RaffleDrawThirdPlace)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if thirdFound and prepareNext then
		if not GLR_Global_Variables[3] then
			local value3 = "true"
			return value3
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function RaffleDrawSecondPlace()
	
	local firstTicket = glrCharacter.Event.Raffle.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Raffle.LastTicketBought - 1
	local secondFound = false
	local prepareNext = false
	local doStep = glrCharacter.Data.Settings.AllowMultipleRaffleWinners
	local continue = true
	local count = 0
	
	for i = 1, #glrCharacter.Raffle.Entries.Names do
		count = count + 1
		if count == 3 then --Don't want to use up too much processing power if their are a lot of entries.
			break
		end
	end
	
	if glrCharacter.Event.Raffle.SecondPlace ~= false then
		if Variables[2][1] then
			for i = 1, #raffleTicketNumbersTable do
				if raffleTicketNumbersTable[i] == Variables[5][2] then
					for t, v in pairs(glrCharacter.Raffle.Entries.Names) do
						for j = 1, #glrCharacter.Raffle.Entries.TicketNumbers[v] do
							local tn = glrCharacter.Raffle.Entries.TicketNumbers[v][j]
							
							if tn == Variables[5][2] then
								
								if count >= 2 and doStep then
									for k = 1, #raffleWinners do
										if raffleWinners[k] == v then
											continue = false
										end
									end
								end
								
								if not GLR_Global_Variables[3] and continue then
									Variables[4][2] = v
									table.insert(raffleWinners, v)
									secondFound = true
									prepareNext = true
									Variables[2][2] = true
									break
								elseif GLR_Global_Variables[3] and not Variables[9] then
									C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
									glrRaffleDrawInProgress = false
									Variables[9] = true
								end
								
							end
						end
						
						if secondFound or not continue then
							break
						end
						
					end
				end
				
				if secondFound or not continue then
					break
				end
				
			end
		end
		
	elseif glrCharacter.Event.Raffle.SecondPlace == false then
		secondFound = true
		prepareNext = true
		Variables[2][2] = true
	end
	
	if not secondFound then
		if not GLR_Global_Variables[3] then
			if Variables[2][1] then --Prevents redrawing a different ticket number while searching for a first place winner.
				Variables[5][2] = GenerateTicket(firstTicket, lastTicket, "Raffle")
			end
			C_Timer.After(1, RaffleDrawSecondPlace)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if secondFound and prepareNext then
		if not GLR_Global_Variables[3] then
			local value2 = "true"
			return value2
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function RaffleDrawFirstPlace()
	
	local firstTicket = glrCharacter.Event.Raffle.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Raffle.LastTicketBought - 1
	local firstFound = false
	local prepareNext = false
	
	for i = 1, #raffleTicketNumbersTable do
		if raffleTicketNumbersTable[i] == Variables[5][1] then
			for t, v in pairs(glrCharacter.Raffle.Entries.Names) do
				for j = 1, #glrCharacter.Raffle.Entries.TicketNumbers[v] do
					
					local tn = glrCharacter.Raffle.Entries.TicketNumbers[v][j]
					
					if tn == Variables[5][1] then
						
						if not GLR_Global_Variables[3] then
							Variables[4][1] = v
							table.insert(raffleWinners, v)
							firstFound = true
							prepareNext = true
							Variables[2][1] = true
							break
						elseif GLR_Global_Variables[3] and not Variables[9] then
							C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
							glrRaffleDrawInProgress = false
							Variables[9] = true
						end
						
					end
				end
				
				if firstFound then
					break
				end
				
			end
		end
		
		if firstFound then
			break
		end
		
	end
	
	if not firstFound then
		if not GLR_Global_Variables[3] then
			Variables[5][1] = GenerateTicket(firstTicket, lastTicket, "Raffle")
			C_Timer.After(1, RaffleDrawFirstPlace)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if firstFound and prepareNext then
		if not GLR_Global_Variables[3] then
			local value1 = "true"
			return value1
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function RaffleCompletionBuffer() --Acts as a buffer while the Mod searches for winning ticket numbers before erasing the raffle data.
	
	if Variables[2][1] then
		if Variables[2][2] then
			if Variables[2][3] then
				if not GLR_Global_Variables[3] then
					if not Variables[8] and not Variables[14] then
						AnnounceRaffleWinners()
					end
					if not Variables[7][2] then
						C_Timer.After(5, RaffleCompletionBuffer)
					end
				end
			end
		end
	end
	
	if not Variables[2][1] or not Variables[2][2] or not Variables[2][3] then
		if not Variables[7][2] then
			if not GLR_Global_Variables[3] then
				C_Timer.After(5, RaffleCompletionBuffer)
			end
		end
	end
	
	if Variables[7][2] then
		if not GLR_Global_Variables[3] then
			C_Timer.After(5, PrepareNextRaffle)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function RaffleStep4Buffer()
	
	if Variables[2][1] and Variables[2][2] and Variables[2][3] then
		RaffleCompletionBuffer()
	else
		C_Timer.After(5, RaffleStep4Buffer)
	end
	
end

local function BeginRaffleDrawStep4()
	
	local firstTicket = glrCharacter.Event.Raffle.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Raffle.LastTicketBought - 1
	local count = glrCharacter.Event.Raffle.LastTicketBought * 10
	local continue = false
	raffleTicketNumbersTable = {} --Clears the table in case another Raffle Roll is started before someone does a /reload
	Variables[1][1] = glrCharacter.Event.Raffle.FirstPlace
	Variables[1][2] = false
	Variables[1][3] = false
	Variables[5][1] = GenerateTicket(firstTicket, lastTicket, "Raffle")
	Variables[5][2] = GenerateTicket(firstTicket, lastTicket, "Raffle")
	Variables[5][3] = GenerateTicket(firstTicket, lastTicket, "Raffle")
	
	if glrCharacter.Event.Raffle.SecondPlace ~= false then
		Variables[1][2] = glrCharacter.Event.Raffle.SecondPlace
	else
		Variables[2][2] = true
	end
	if glrCharacter.Event.Raffle.ThirdPlace ~= false then
		Variables[1][3] = glrCharacter.Event.Raffle.ThirdPlace
	else
		Variables[2][3] = true
	end
	
	for t, v in pairs(glrCharacter.Raffle.Entries.Names) do
		for i = 1, #glrCharacter.Raffle.Entries.TicketNumbers[v] do
			table.insert(raffleTicketNumbersTable, glrCharacter.Raffle.Entries.TicketNumbers[v][i])
		end
	end
	
	sort(raffleTicketNumbersTable)
	
	if not GLR_Global_Variables[3] then
		
		local value1 = RaffleDrawFirstPlace()
		local value2 = "true"
		local value3 = "true"
		
		if glrCharacter.Event.Raffle.SecondPlace ~= false then
			value2 = RaffleDrawSecondPlace()
		end
		if glrCharacter.Event.Raffle.ThirdPlace ~= false then
			value3 = RaffleDrawThirdPlace()
		end
		
		RaffleStep4Buffer()
		
	elseif GLR_Global_Variables[3] and not Variables[9] then
		
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrRaffleDrawInProgress = false
		Variables[9] = true
		
	end
	
end

local function AdvancedRaffleDrawWinners()
	
	if not glrCharacter.Data.Settings.InverseAnnounce then --Announce 1st -> 2nd -> 3rd
		if not Variables[3][1] then
			C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce First Ticket"] .. Variables[5][1], "GUILD") end)
			C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 First Winner Found"] .. Variables[4][1] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][1], "GUILD") end)
			C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[3][1] = true
			C_Timer.After(7, AdvancedRaffleDrawWinners)
		elseif not Variables[3][2] then
			if glrCharacter.Event.Raffle.SecondPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Second Ticket"] .. Variables[5][2], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Second Winner Found"] .. Variables[4][2] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][2], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(7, AdvancedRaffleDrawWinners)
			else
				C_Timer.After(2, AdvancedRaffleDrawWinners)
			end
			Variables[3][2] = true
		elseif not Variables[3][3] then
			if glrCharacter.Event.Raffle.ThirdPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Third Ticket"] .. Variables[5][3], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Third Winner Found"] .. Variables[4][3] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][3], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			end
			Variables[7][2] = true
			Variables[3][3] = true
		end
	else
		if not Variables[3][3] then
			if glrCharacter.Event.Raffle.ThirdPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Third Ticket"] .. Variables[5][3], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Third Winner Found"] .. Variables[4][3] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][3], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(7, AdvancedRaffleDrawWinners)
			else
				C_Timer.After(2, AdvancedRaffleDrawWinners)
			end
			Variables[3][3] = true
		elseif not Variables[3][2] then
			if glrCharacter.Event.Raffle.SecondPlace ~= false then
				C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce Second Ticket"] .. Variables[5][2], "GUILD") end)
				C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 Second Winner Found"] .. Variables[4][2] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][2], "GUILD") end)
				C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(7, AdvancedRaffleDrawWinners)
			else
				C_Timer.After(2, AdvancedRaffleDrawWinners)
			end
			Variables[3][2] = true
		elseif not Variables[3][1] then
			C_Timer.After(0.5, function(self) SendChatMessage(L["Raffle Draw Step 5 Announce First Ticket"] .. Variables[5][1], "GUILD") end)
			C_Timer.After(1, function(self) SendChatMessage(L["Raffle Draw Step 5 First Winner Found"] .. Variables[4][1] .. L["Raffle Draw Step 5 Winner Found Fixed Variable"] .. Variables[1][1], "GUILD") end)
			C_Timer.After(1.5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[3][1] = true
			Variables[7][2] = true
		end
	end
	
	if Variables[7][2] then
		if not GLR_Global_Variables[3] then
			C_Timer.After(5, PrepareNextRaffle)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrRaffleDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function AdvancedRaffleDrawBuffer() --Buffer while the AddOn generates the Winning Ticket(s)
	if Variables[16] then
		Variables[4][1] = RandomizationTable.Numbers[Variables[5][1]]
		if glrCharacter.Event.Raffle.SecondPlace ~= false then
			Variables[4][2] = RandomizationTable.Numbers[Variables[5][2]]
		end
		if glrCharacter.Event.Raffle.ThirdPlace ~= false then
			Variables[4][3] = RandomizationTable.Numbers[Variables[5][3]]
		end
		for t, v in pairs(RandomizationTable.Names) do --Transfers the randomly given ticket numbers from memory to the save data file.
			local low, high = FindNumberRange(t)
			glrCharacter.Raffle.Entries.TicketRange[t].LowestTicketNumber = low
			glrCharacter.Raffle.Entries.TicketRange[t].HighestTicketNumber = high
			glrCharacter.Raffle.Entries.TicketNumbers[t] = RandomizationTable.Names[t]
		end
		C_Timer.After(5, AdvancedRaffleDrawWinners)
	else
		C_Timer.After(1, AdvancedRaffleDrawBuffer)
	end
end

local function AdvancedRaffleDrawTickets()
	
	local first = glrCharacter.Event.Raffle.FirstTicketNumber
	local last = glrCharacter.Event.Raffle.LastTicketBought - 1
	local continue = false
	local check = false
	Variables[1][1] = glrCharacter.Event.Raffle.FirstPlace
	Variables[5][1] = tostring(math.random(first, last))
	
	if glrCharacter.Event.Raffle.SecondPlace ~= false then
		Variables[1][2] = glrCharacter.Event.Raffle.SecondPlace
		Variables[5][2] = tostring(math.random(first, last))
	else
		Variables[2][2] = true
	end
	if glrCharacter.Event.Raffle.ThirdPlace ~= false then
		Variables[1][3] = glrCharacter.Event.Raffle.ThirdPlace
		Variables[5][3] = tostring(math.random(first, last))
	else
		Variables[2][3] = true
	end
	
	while true do --Generates up to three different Winning Ticket Numbers.
		if glrCharacter.Event.Raffle.SecondPlace ~= false then
			if Variables[17] == 1 then break end --Only one total Ticket was sold/given so why check since ticket numbers will be the same for two prizes?
			if Variables[5][1] == Variables[5][2] then
				Variables[5][2] = tostring(math.random(first, last))
			end
			if glrCharacter.Event.Raffle.ThirdPlace ~= false then
				if Variables[5][1] == Variables[5][3] then
					Variables[5][3] = tostring(math.random(first, last))
				end
				if Variables[5][2] == Variables[5][3] then
					Variables[5][3] = tostring(math.random(first, last))
				end
				if Variables[5][1] ~= Variables[5][2] and Variables[5][1] ~= Variables[5][3] and Variables[5][2] ~= Variables[5][3] or Variables[17] == 2 then continue = true break end
			else
				if Variables[5][1] ~= Variables[5][2] then continue = true break end
			end
		else continue = true break end
	end
	
	if glrCharacter.Data.Settings.AllowMultipleRaffleWinners then
		local total = #glrCharacter.Raffle.Entries.Names
		local FP = RandomizationTable.Numbers[Variables[5][1]]
		local SP = "GLR_2"
		local TP = "GLR_3"
		local CT = 1
		if glrCharacter.Event.Raffle.SecondPlace ~= false then
			SP = RandomizationTable.Numbers[Variables[5][2]]
			CT = CT + 1
		end
		if glrCharacter.Event.Raffle.ThirdPlace ~= false then
			TP = RandomizationTable.Numbers[Variables[5][3]]
			CT = CT + 1
		end
		while true do
			if SP ~= "GLR_2" then
				if FP == SP then
					Variables[5][2] = tostring(math.random(first, last))
					SP = RandomizationTable.Numbers[Variables[5][2]]
				end
			end
			if TP ~= "GLR_3" then
				if FP == TP or SP == TP then
					Variables[5][3] = tostring(math.random(first, last))
					TP = RandomizationTable.Numbers[Variables[5][3]]
				end
			end
			if total < CT then check = true break end
			if FP ~= SP and FP ~= TP and SP ~= TP then check = true break end
		end
	else check = true end
	
	if continue and check then Variables[16] = true end
	
	if not GLR_Global_Variables[3] then
		AdvancedRaffleDrawBuffer()
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrRaffleDrawInProgress = false
		Variables[9] = true
	end
	
end

local function RaffleDrawAnnounceRoll()
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 4 Announce Roll"], "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
		if not Variables[14] then
			C_Timer.After(10.6, BeginRaffleDrawStep4)
		else
			C_Timer.After(10.6, AdvancedRaffleDrawTickets)
		end
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrRaffleDrawInProgress = false
		Variables[9] = true
	end
	
end

local function BeginRaffleDrawStep2()
	
	local ticketPool = {}
	local numberOfInvalidTickets = 0
	local lowestTicketNumber = glrCharacter.Event.Raffle.FirstTicketNumber
	local highestTicketNumber = glrCharacter.Event.Raffle.LastTicketBought - 1
	
	if glrCharacter.Raffle.TicketPool ~= nil then
		for t, v in pairs(glrCharacter.Raffle.TicketPool) do
			numberOfInvalidTickets = numberOfInvalidTickets + 1
		end
	end
	
	if numberOfInvalidTickets > 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(string.gsub(L["Draw Step 2 Invalid Tickets"], "%%unsold", numberOfInvalidTickets), "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(string.gsub(string.gsub(L["Draw Step 2"], "%%low", lowestTicketNumber), "%%high", highestTicketNumber), "GUILD") end)
	else
		C_Timer.After(0.2, function(self) SendChatMessage(string.gsub(string.gsub(L["Draw Step 2"], "%%low", lowestTicketNumber), "%%high", highestTicketNumber), "GUILD") end)
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(12, RaffleDrawAnnounceRoll)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrRaffleDrawInProgress = false
		Variables[9] = true
	end
	
end

local function AdvancedRaffleDraw()
	
	local numberOfTickets = glrCharacter.Event.Raffle.TicketsSold
	local jackpot = tostring(glrCharacter.Event.Raffle.TicketsSold * tonumber(glrCharacter.Event.Raffle.TicketPrice))
	local first = glrCharacter.Event.Raffle.FirstTicketNumber
	local last = glrCharacter.Event.Raffle.LastTicketBought - 1
	local n = first
	local c = 0
	local c2 = 0
	local firstP = glrCharacter.Event.Raffle.FirstPlace
	local secondP = false
	local thirdP = false
	Variables[17] = glrCharacter.Event.Raffle.TicketsSold + glrCharacter.Event.Raffle.GiveAwayTickets
	
	local JackpotValue = GetValue(jackpot, "Lottery", "Jackpot", false)
	
	if glrCharacter.Event.Raffle.SecondPlace ~= false then
		secondP = glrCharacter.Event.Raffle.SecondPlace
	end
	if glrCharacter.Event.Raffle.ThirdPlace ~= false then
		thirdP = glrCharacter.Event.Raffle.ThirdPlace
	end
	
	for i = 1, Variables[17] do
		table.insert(EventTickets, tostring(n))
		Variables[13][2] = Variables[13][2] + 1
		n = n + 1
		c = c + 1
	end
	
	RandomizationTable.Totals = EventTickets
	
	for i = 1, #glrCharacter.Raffle.Entries.Names do
		for j = 1, glrCharacter.Raffle.Entries.Tickets[glrCharacter.Raffle.Entries.Names[i]].NumberOfTickets do
			local t = math.random(1, #EventTickets)
			if RandomizationTable.Names[glrCharacter.Raffle.Entries.Names[i]] == nil then
				RandomizationTable.Names[glrCharacter.Raffle.Entries.Names[i]] = {}
			end
			table.insert(RandomizationTable.Names[glrCharacter.Raffle.Entries.Names[i]], EventTickets[t])
			RandomizationTable.Numbers[EventTickets[t]] = glrCharacter.Raffle.Entries.Names[i]
			table.remove(EventTickets, t)
			c2 = c2 + 1
		end
	end
	
	local ms = 0.3
	
	SendChatMessage(string.gsub(string.gsub(L["Draw Step 1"], "%%TicketsSold", numberOfTickets), "%%money", JackpotValue), "GUILD")
	
	if glrCharacter.Event.Raffle.GiveAwayTickets ~= nil then
		if glrCharacter.Event.Raffle.GiveAwayTickets > 0 then
			local GivenTickets = tostring(glrCharacter.Event.Raffle.GiveAwayTickets)
			local message = L["Send Info GLR"] .. ": " .. string.gsub(L["Draw Given Tickets"], "%%t", GivenTickets)
			C_Timer.After(ms, function(self) SendChatMessage(message, "GUILD") end)
			ms = ms + 0.3
		end
	end
	
	C_Timer.After(ms, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 1"], "%%p1", firstP), "GUILD") end)
	ms = ms + 0.3
	
	if secondP ~= false then
		C_Timer.After(ms, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 2"], "%%p2", secondP), "GUILD") end)
		if thirdP ~= false then
			ms = ms + 0.3
		end
	end
	if thirdP ~= false then
		C_Timer.After(ms, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 3"], "%%p3", thirdP), "GUILD") end)
	end
	
	if not GLR_Global_Variables[3] then
		local message, _ = string.gsub(string.gsub(L["Advanced Draw Step 1"], "%%f", first), "%%l", last)
		local tickets, _ = string.gsub(string.gsub(L["Advanced Draw Step 1 Totals"], "%%t", c), "%%c", c2)
		C_Timer.After(8, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
		C_Timer.After(8.3, function(self) SendChatMessage(tickets, "GUILD") end)
		C_Timer.After(8.6, function(self) SendChatMessage(message, "GUILD") end)
		
		C_Timer.After(9, RaffleDrawAnnounceRoll)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function BeginRaffleDrawStep1()
	
	local jackpotGold = tostring(glrCharacter.Event.Raffle.TicketsSold * tonumber(glrCharacter.Event.Raffle.TicketPrice))
	local numberOfTickets = glrCharacter.Event.Raffle.TicketsSold
	Variables[1][1] = glrCharacter.Event.Raffle.FirstPlace
	Variables[1][2] = false
	Variables[1][3] = false
	
	local JackpotValue = GetValue(jackpotGold, "Lottery", "Jackpot", false)
	
	if glrCharacter.Event.Raffle.SecondPlace ~= false then
		Variables[1][2] = glrCharacter.Event.Raffle.SecondPlace
	end
	if glrCharacter.Event.Raffle.ThirdPlace ~= false then
		Variables[1][3] = glrCharacter.Event.Raffle.ThirdPlace
	end
	
	C_Timer.After(0.2, function(self) SendChatMessage(string.gsub(string.gsub(L["Draw Step 1"], "%%TicketsSold", numberOfTickets), "%%money", JackpotValue), "GUILD") end)
	
	if glrCharacter.Event.Raffle.GiveAwayTickets ~= nil then
		if glrCharacter.Event.Raffle.GiveAwayTickets > 0 then
			local GivenTickets = tostring(glrCharacter.Event.Raffle.GiveAwayTickets)
			C_Timer.After(0.4, function(self) SendChatMessage(L["Send Info GLR"] .. ": " .. string.gsub(L["Draw Given Tickets"], "%%t", GivenTickets), "GUILD") end)
		end
	end
	
	C_Timer.After(0.6, function(self) SendChatMessage(L["Raffle Draw Step 1"] .. ":", "GUILD") end)
	C_Timer.After(0.8, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 1"], "%%p1", Variables[1][1]), "GUILD") end)
	
	if Variables[1][2] ~= false then
		C_Timer.After(1, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 2"], "%%p2", Variables[1][2]), "GUILD") end)
	end
	if Variables[1][3] ~= false then
		C_Timer.After(1.2, function(self) SendChatMessage(string.gsub(L["Raffle Draw Step 1 Prize 3"], "%%p3", Variables[1][3]), "GUILD") end)
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(13.2, BeginRaffleDrawStep2)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrRaffleDrawInProgress = false
		Variables[9] = true
	end
	
end

local function InitiateRaffleRoll()
	
	local raffleStatus = glrCharacter.Data.Settings.CurrentlyActiveRaffle
	Variables[14] = glrCharacter.Data.Settings.AdvancedTicketDraw
	RandomizationTable = { ["Names"] = {}, ["Numbers"] = {}, }
	Variables[17] = 0
	EventTickets = {}
	Variables[2][1] = false
	Variables[2][2] = false
	Variables[2][3] = false
	Variables[4][1] = ""
	Variables[4][2] = ""
	Variables[4][3] = ""
	Variables[3][1] = false
	Variables[3][2] = false
	Variables[3][3] = false
	Variables[8] = false
	Variables[7][2] = false
	GLR_Global_Variables[3] = false
	Variables[9] = false
	raffleWinners = {} --Resets the table if entries exist
	
	if raffleStatus and glrCharacter.Event.Raffle ~= nil then --Prevents starting a non-existent raffle draw
		if glrCharacter.Event.Raffle.TicketsSold >= 1 or glrCharacter.Event.Raffle.GiveAwayTickets >= 1 then
			
			GLR_UI.GLR_MainFrame.GLR_AbortRollProcessButton:Show()
			
			if not GLR_Global_Variables[3] then
				glrRaffleDrawInProgress = true --Prevents a raffle draw from occuring while one's in progress
				SendChatMessage("--------------------------------------------------", "GUILD")
				C_Timer.After(0.3, function(self) SendChatMessage(L["GLR Title with Version"] .. GLR_CurrentVersionString, "GUILD") end)
				if not Variables[14] then
					C_Timer.After(0.6, function(self) SendChatMessage(L["Begin Raffle Draw"], "GUILD") end)
				else
					C_Timer.After(0.6, function(self) SendChatMessage(L["Begin Advanced Raffle Draw"], "GUILD") end)
				end
				C_Timer.After(0.9, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				if not Variables[14] then
					C_Timer.After(15, BeginRaffleDrawStep1)
				else
					C_Timer.After(15, AdvancedRaffleDraw)
				end
			elseif GLR_Global_Variables[3] and not Variables[9] then
				C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
				glrRaffleDrawInProgress = false
				Variables[9] = true
			end
			
		end
		
	else
		GLR:Print(L["No Raffle Active"])
	end
	
end

---------------------------------------------
------------- STARTS THE LOTTERY ------------
---------------------------------------------
-------- STARTS LOTTERY, EVEN BEFORE --------
----------- THE SET LOTTERY DATE ------------
---------------------------------------------

local function LotteryCompletionBuffer() --Acts as a buffer while the Mod searches for winning ticket numbers before erasing the Lottery data.
	
	local timestamp = GetTimeStamp()
	
	if not Variables[18] then
		Variables[18] = true
		table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryCompletionBuffer() - Started Buffer Function")
	end
	
	if Variables[10] then
		if not GLR_Global_Variables[3] then
			Variables[7][1] = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if not Variables[10] then
		if not Variables[7][1] then
			if not GLR_Global_Variables[3] then
				C_Timer.After(5, LotteryCompletionBuffer)
			elseif GLR_Global_Variables[3] and not Variables[9] then
				C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
				glrLotteryDrawInProgress = false
				Variables[9] = true
			end
		end
	end
	
	if Variables[7][1] then
		table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryCompletionBuffer() - Buffer Function Complete")
		if not GLR_Global_Variables[3] then
			C_Timer.After(7, PrepareNextLottery)
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function LotteryDrawSearchSecondPlace() --Only does this step if no winner is guaranteed and no one won first place.
	
	local firstTicket = glrCharacter.Event.Lottery.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Lottery.LastTicketBought
	local winningTicket = tostring(math.random(firstTicket, lastTicket))
	local jackpotGold = tonumber(glrCharacter.Event["Lottery"]["StartingGold"]) + ( glrCharacter.Event["Lottery"]["TicketsSold"] * tonumber(glrCharacter.Event["Lottery"]["TicketPrice"]) )
	local guildPercent = (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100) * jackpotGold
	local secondPercent = (tonumber(glrCharacter.Event.Lottery.SecondPlace) / 100) * jackpotGold
	local round = glrCharacter.Data.Settings.RoundJackpot
	local ticketNumbersTable = {}
	local ticketPool = {}
	local winnerFound = false
	local prepareNext = false
	local redraw = false
	Variables[12] = false
	Variables[13][1] = Variables[13][1] - secondPercent
	Variables[6] = winningTicket
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryDrawSearchSecondPlace() - Searching for Second Place Winner. Winning Ticket: " .. winningTicket)
	
	for t, v in pairs(glrCharacter.Lottery.Entries.Names) do
		for i = 1, #glrCharacter.Lottery.Entries.TicketNumbers[v] do
			table.insert(ticketNumbersTable, glrCharacter.Lottery.Entries.TicketNumbers[v][i])
		end
	end
	
	for t, v in pairs(glrCharacter.Lottery.TicketPool) do
		table.insert(ticketPool, v)
	end
	
	local remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
	local secondValue = GetValue(secondPercent, "Lottery", "Jackpot", round)
	local guildValue = GetValue(guildPercent, "Lottery", "Jackpot", round)
	
	for i = 1, #ticketNumbersTable do
		if ticketNumbersTable[i] == winningTicket then
			for t, v in pairs(glrCharacter.Lottery.Entries.Names) do
				for j = 1, #glrCharacter.Lottery.Entries.TicketNumbers[v] do
					local tn = glrCharacter.Lottery.Entries.TicketNumbers[v][j]
					if tn == winningTicket then
						if not GLR_Global_Variables[3] then
							C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
							C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Announce Second Place Winning Ticket"], "%%t", winningTicket), "GUILD") end)
							C_Timer.After(3, function(self) SendChatMessage(string.gsub(string.gsub(L["Lottery Draw Step 6 Winner Found"], "%%v", v), "%%g", secondValue), "GUILD") end)
							C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
							C_Timer.After(5, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Next Lottery Starts"], "%%g", remainingValue), "GUILD") end)
							C_Timer.After(6, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
							Variables[10] = true
							winnerFound = true
							prepareNext = true
						elseif GLR_Global_Variables[3] and not Variables[9] then
							C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
							glrLotteryDrawInProgress = false
							Variables[9] = true
						end
					end
				end
			end
		end
	end
	
	if not winnerFound then
		for i = 1, #ticketPool do --If the winning ticket number wasn't assigned to a person, it redoes the ticket draw.
			if ticketPool[i] == winningTicket then
				if not GLR_Global_Variables[3] then
					redraw = true
					C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
					C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Unused Ticket Found"], "%%t", winningTicket), "GUILD") end)
					C_Timer.After(3, function(self) SendChatMessage(L["Lottery Draw Unused Ticket Redraw"], "GUILD") end)
					C_Timer.After(13, LotteryDrawSearchSecondPlace)
					winnerFound = true
				elseif GLR_Global_Variables[3] and not Variables[9] then
					C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
					glrLotteryDrawInProgress = false
					Variables[9] = true
				end
			end
		end
	end
	
	if not winnerFound and not redraw then
		Variables[12] = true --Prevents the %previous_jackpot variable from showing the second place value of the Jackpot since no one won.
		if not GLR_Global_Variables[3] then
			if glrCharacter.Data.Settings.NoGuildCut then
				guildPercent = 0
			end
			Variables[13][1] = jackpotGold - guildPercent
			remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
			C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 No Winner Found"], "%%t", winningTicket), "GUILD") end)
			C_Timer.After(3, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 No Second Place"], "%%g", remainingValue), "GUILD") end)
			C_Timer.After(4, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[10] = true
			prepareNext = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if prepareNext then
		if not GLR_Global_Variables[3] then
			LotteryCompletionBuffer()
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function LotteryDrawSearchFirstPlace(winningTicket) --Searches for a valid First Place Winner.
	
	local jackpotGold = tonumber(glrCharacter.Event["Lottery"]["StartingGold"]) + ( glrCharacter.Event["Lottery"]["TicketsSold"] * tonumber(glrCharacter.Event["Lottery"]["TicketPrice"]) )
	local guildPercent = (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100) * jackpotGold
	local round = glrCharacter.Data.Settings.RoundJackpot
	local winnerFound = false
	local prepareNext = false
	local bypass = false
	local ticketNumbersTable = {}
	local secondPercent = 1
	if tonumber(glrCharacter.Event.Lottery.SecondPlace) ~= nil then
		secondPercent = tonumber(glrCharacter.Event.Lottery.SecondPlace)
	end
	if glrCharacter.Event.Lottery.SecondPlace ~= L["Winner Guaranteed"] then
		if tonumber(glrCharacter.Event.Lottery.SecondPlace) == 0 then
			bypass = true -- Since Second Place is 0% of the Jackpot, why attempt a draw for it?
		end
	elseif glrCharacter.Event.Lottery.SecondPlace == L["Winner Guaranteed"] then
		winnerFound = true --Prevents the mod from attempting to draw a second place ticket since a winner is guaranteed
	end
	
	Variables[6] = winningTicket
	
	for t, v in pairs(glrCharacter.Lottery.Entries.Names) do
		for i = 1, #glrCharacter.Lottery.Entries.TicketNumbers[v] do
			table.insert(ticketNumbersTable, glrCharacter.Lottery.Entries.TicketNumbers[v][i])
		end
	end
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryDrawSearchFirstPlace() - Searching for First Place Winner. Winning Ticket: " .. winningTicket .. " Is Winner Guaranteed? " .. string.upper(tostring(bypass)) )
	
	sort(ticketNumbersTable)
	
	local remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
	local guildValue = GetValue(guildPercent, "Lottery", "Jackpot", round)
	
	Variables[12] = true
	
	if secondPercent > 0 then
		for i = 1, #ticketNumbersTable do
			if ticketNumbersTable[i] == winningTicket then
				for t, v in pairs(glrCharacter.Lottery.Entries.Names) do
					for j = 1, #glrCharacter.Lottery.Entries.TicketNumbers[v] do
						local tn = glrCharacter.Lottery.Entries.TicketNumbers[v][j]
						if tn == winningTicket then
							if not GLR_Global_Variables[3] then
								C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
								C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 Announce Winning Ticket"], "%%t", winningTicket), "GUILD") end)
								C_Timer.After(3, function(self) SendChatMessage(string.gsub(string.gsub(L["Lottery Draw Step 5 Winner Found"], "%%v", v), "%%g", remainingValue), "GUILD") end)
								C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
								C_Timer.After(5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
								Variables[11] = true
								Variables[10] = true
								winnerFound = true
								prepareNext = true
							elseif GLR_Global_Variables[3] and not Variables[9] then
								C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
								glrLotteryDrawInProgress = false
								Variables[9] = true
							end
						end
					end
				end
			end
		end
	end
	
	if not winnerFound and secondPercent == 0 then --Only does this if no Winner Guaranteed, Second Place % is zero
		if not GLR_Global_Variables[3] then
			for i = 1, #ticketNumbersTable do
				if ticketNumbersTable[i] == winningTicket then
					for t, v in pairs(glrCharacter.Lottery.Entries.Names) do
						for j = 1, #glrCharacter.Lottery.Entries.TicketNumbers[v] do
							local tn = glrCharacter.Lottery.Entries.TicketNumbers[v][j]
							if tn == winningTicket then
								C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
								C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 Announce Winning Ticket"], "%%t", winningTicket), "GUILD") end)
								C_Timer.After(3, function(self) SendChatMessage(string.gsub(string.gsub(L["Lottery Draw Step 5 Winner Found"], "%%v", v), "%%g", remainingValue), "GUILD") end)
								C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
								C_Timer.After(5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
								Variables[11] = true
								Variables[10] = true
								winnerFound = true
								prepareNext = true
							end
						end
					end
				end
			end
			if not Variables[10] then
				C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 No Winner Found"], "%%t", winningTicket), "GUILD") end)
				if glrCharacter.Data.Settings.NoGuildCut then
					guildValue = 0
					Variables[13][1] = jackpotGold
				end
				C_Timer.After(3, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
				C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Next Lottery Starts"], "%%g", remainingValue), "GUILD") end)
				Variables[10] = true
				winnerFound = true
				prepareNext = true
			end
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if not winnerFound then --Only does this if no one won the Jackpot but someone can win Second Place.
		if not bypass then
			if not GLR_Global_Variables[3] then
				C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 No Winner Found"], "%%t", winningTicket), "GUILD") end)
				C_Timer.After(3, function(self) SendChatMessage(L["Lottery Draw Step 5 Draw Second Place"], "GUILD") end)
				C_Timer.After(13, LotteryDrawSearchSecondPlace)
			elseif GLR_Global_Variables[3] and not Variables[9] then
				C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
				glrLotteryDrawInProgress = false
				Variables[9] = true
			end
		end
	end
	
	if prepareNext then
		if not GLR_Global_Variables[3] then
			LotteryCompletionBuffer()
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
end

local function LotteryDrawSearchInvalidTickets() --Is the first part of the Event Draw where the Winning Ticket number is generated. This allows the mod to search for potentially invalid tickets (one's that were sold but taken back and therefore still generated)
	
	local firstTicket = glrCharacter.Event.Lottery.FirstTicketNumber
	local lastTicket = glrCharacter.Event.Lottery.LastTicketBought - 1
	local ticketPool = {}
	local continue = false
	local ticketNotFound = true
	local count = 0
	local winningTicket = tostring(math.random(firstTicket, lastTicket))
	
	local timestamp = GetTimeStamp()
	
	for t, v in pairs(glrCharacter.Lottery.TicketPool) do
		table.insert(ticketPool, v)
		count = count + 1
	end
	
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryDrawSearchInvalidTickets() - Searching for Invalid Tickets. Winning Ticket: " .. winningTicket .. " Invalid Ticket Count: " .. count)
	
	if count == 0 then
		continue = true
	end
	
	if count >= 1 then
		for i = 1, #ticketPool do --If the winning ticket number wasn't assigned to a person, it redoes the ticket draw.
			count = count - 1
			if ticketPool[i] == winningTicket then
				if not GLR_Global_Variables[3] then
					ticketNotFound = false
					C_Timer.After(0.2, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
					C_Timer.After(0.4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Unused Ticket Found"], "%%t", winningTicket), "GUILD") end)
					C_Timer.After(0.6, function(self) SendChatMessage(L["Lottery Draw Unused Ticket Redraw"], "GUILD") end)
					C_Timer.After(10, LotteryDrawSearchInvalidTickets)
				elseif GLR_Global_Variables[3] and not Variables[9] then
					C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
					glrLotteryDrawInProgress = false
					Variables[9] = true
				end
			end
			if count == 0 then
				continue = true
			end
		end
	end
	
	if count == 0 and continue and ticketNotFound then
		LotteryDrawSearchFirstPlace(winningTicket)
	end
	
end

local function AdvancedLotteryFindSecond(first, last) --Will only do this if no one won the Jackpot but someone has a chance at Second Place
	
	local jackpotGold = tonumber(glrCharacter.Event.Lottery.StartingGold) + ( glrCharacter.Event.Lottery.TicketsSold * tonumber(glrCharacter.Event.Lottery.TicketPrice) )
	local guildPercent = (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100) * jackpotGold
	local secondPercent = (tonumber(glrCharacter.Event.Lottery.SecondPlace) / 100) * jackpotGold
	local round = glrCharacter.Data.Settings.RoundJackpot
	local winnerFound = false
	local prepareNext = false
	local found = false
	local winner
	local ticket = tostring(math.random(first, last))
	local remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
	local secondValue = GetValue(secondPercent, "Lottery", "Jackpot", round)
	local guildValue = GetValue(guildPercent, "Lottery", "Jackpot", round)
	
	Variables[12] = false -- Denotes a First Place winner wasn't found in the Previous Event data table.
	
	while true do -- Prevents the AddOn from drawing the same Winning Ticket Number for Second Place as was selected for the Jackpot.
		if ticket == Variables[15] then
			ticket = tostring(math.random(first, last))
		else break end
	end
	
	Variables[6] = ticket
	
	if RandomizationTable.Numbers[ticket] ~= nil then
		winner = RandomizationTable.Numbers[ticket]
		found = true
	end
	
	if found then
		if not GLR_Global_Variables[3] then
			C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Announce Second Place Winning Ticket"], "%%t", ticket), "GUILD") end)
			C_Timer.After(3, function(self) SendChatMessage(string.gsub(string.gsub(L["Lottery Draw Step 6 Winner Found"], "%%v", winner), "%%g", secondValue), "GUILD") end)
			C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
			C_Timer.After(5, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Next Lottery Starts"], "%%g", remainingValue), "GUILD") end)
			C_Timer.After(6, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[10] = true
			winnerFound = true
			prepareNext = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if not winnerFound then
		if not GLR_Global_Variables[3] then
			if glrCharacter.Data.Settings.NoGuildCut then
				guildPercent = 0
			end
			Variables[13][1] = jackpotGold - guildPercent
			remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
			C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 No Winner Found"], "%%t", ticket), "GUILD") end)
			C_Timer.After(3, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 No Second Place"], "%%g", remainingValue), "GUILD") end)
			C_Timer.After(4, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[10] = true
			prepareNext = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - AdvancedLotteryFindSecond() - Second Place Found? " .. string.upper(tostring(found)) .. " - Transfered Winning Ticket Number: " .. Variables[6])
	
	if prepareNext then
		if not GLR_Global_Variables[3] then
			LotteryCompletionBuffer()
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function AdvancedLotteryFindWinner()
	
	for t, v in pairs(RandomizationTable.Names) do --Transfers the randomly given ticket numbers from memory to the save data file.
		local low, high = FindNumberRange(t)
		glrCharacter.Lottery.Entries.TicketRange[t].LowestTicketNumber = low
		glrCharacter.Lottery.Entries.TicketRange[t].HighestTicketNumber = high
		glrCharacter.Lottery.Entries.TicketNumbers[t] = RandomizationTable.Names[t]
	end
	
	local found = false
	local first = glrCharacter.Event.Lottery.FirstTicketNumber
	local last = glrCharacter.Event.Lottery.LastTicketBought - 1
	local ticket = tostring(math.random(first, last))
	local winner
	local winnerFound = false
	local jackpotGold = tonumber(glrCharacter.Event.Lottery.StartingGold) + ( glrCharacter.Event.Lottery.TicketsSold * tonumber(glrCharacter.Event.Lottery.TicketPrice) )
	local guildPercent = (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100) * jackpotGold
	local round = glrCharacter.Data.Settings.RoundJackpot
	local bypass = false
	local noSecond = false
	local secondPercent = 1
	if tonumber(glrCharacter.Event.Lottery.SecondPlace) ~= nil then
		secondPercent = tonumber(glrCharacter.Event.Lottery.SecondPlace)
	end
	
	if glrCharacter.Event.Lottery.SecondPlace ~= L["Winner Guaranteed"] then
		if tonumber(glrCharacter.Event.Lottery.SecondPlace) == 0 then
			bypass = true -- Since Second Place is 0% of the Jackpot, why attempt a draw for it?
		end
	elseif glrCharacter.Event.Lottery.SecondPlace == L["Winner Guaranteed"] then
		noSecond = true --Prevents the mod from attempting to draw a second place ticket since a winner is guaranteed
	end
	
	local remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", round)
	local guildValue = GetValue(guildPercent, "Lottery", "Jackpot", round)
	
	if RandomizationTable.Numbers[ticket] ~= nil then
		winner = RandomizationTable.Numbers[ticket]
		found = true
		Variables[12] = true
	end
	
	Variables[6] = ticket
	Variables[15] = ticket
	
	if found then
		if not GLR_Global_Variables[3] then
			C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 Announce Winning Ticket"], "%%t", ticket), "GUILD") end)
			C_Timer.After(3, function(self) SendChatMessage(string.gsub(string.gsub(L["Lottery Draw Step 5 Winner Found"], "%%v", winner), "%%g", remainingValue), "GUILD") end)
			C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
			C_Timer.After(5, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			Variables[11] = true
			Variables[10] = true
			winnerFound = true
			prepareNext = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if not Variables[10] and secondPercent == 0 then --Only does this if no Winner Guaranteed, Second Place % is zero
		if not GLR_Global_Variables[3] then
			C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
			C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 No Winner Found"], "%%t", ticket), "GUILD") end)
			if glrCharacter.Data.Settings.NoGuildCut then
				guildValue = 0
				Variables[13][1] = jackpotGold
			end
			C_Timer.After(3, function(self) SendChatMessage(string.gsub(L["Lottery Draw Guilds Take"], "%%g", guildValue), "GUILD") end)
			C_Timer.After(4, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 6 Next Lottery Starts"], "%%g", remainingValue), "GUILD") end)
			Variables[10] = true
			winnerFound = true
			prepareNext = true
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
	if not winnerFound then --Only does this if no one won the Jackpot but someone can win Second Place.
		if not bypass then
			if not GLR_Global_Variables[3] then
				C_Timer.After(1, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				C_Timer.After(2, function(self) SendChatMessage(string.gsub(L["Lottery Draw Step 5 No Winner Found"], "%%t", ticket), "GUILD") end)
				C_Timer.After(3, function(self) SendChatMessage(L["Lottery Draw Step 5 Draw Second Place"], "GUILD") end)
				C_Timer.After(13, function(self) AdvancedLotteryFindSecond(first, last) end)
			elseif GLR_Global_Variables[3] and not Variables[9] then
				C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
				glrLotteryDrawInProgress = false
				Variables[9] = true
			end
		end
	end
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - AdvancedLotteryFindWinner() - Winner Found? " .. string.upper(tostring(found)) .. " - Transfered Winning Ticket Number: " .. Variables[6])
	
	if prepareNext then
		if not GLR_Global_Variables[3] then
			LotteryCompletionBuffer()
		elseif GLR_Global_Variables[3] and not Variables[9] then
			C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
			glrLotteryDrawInProgress = false
			Variables[9] = true
		end
	end
	
end

local function BeginLotteryDrawStep4() --Announces the amount the Guild and Second Place will receive.
	
	local secondPercent = glrCharacter.Event.Lottery.SecondPlace
	local jackpotGold = tonumber(glrCharacter.Event["Lottery"]["StartingGold"]) + ( glrCharacter.Event["Lottery"]["TicketsSold"] * tonumber(glrCharacter.Event["Lottery"]["TicketPrice"]) )
	local guildPercent = (tonumber(glrCharacter.Event.Lottery.GuildCut) / 100) * jackpotGold
	local GuildCutStatus = glrCharacter.Data.Settings.NoGuildCut
	local round = glrCharacter.Data.Settings.RoundJackpot
	local secondValue = ""
	local ms = 0.2
	Variables[13][1] = jackpotGold - guildPercent
	
	if secondPercent ~= L["Winner Guaranteed"] then
		secondPercent = (tonumber(secondPercent) / 100) * jackpotGold
		secondValue = GetValue(secondPercent, "Lottery", "Jackpot", round)
	end
	
	local remainingValue = GetValue(Variables[13][1], "Lottery", "Jackpot", false)
	local guildValue = GetValue(guildPercent, "Lottery", "Jackpot", round)
	
	if secondPercent == L["Winner Guaranteed"] and guildPercent > 0 then
		C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 Condition 1 Part 1"] .. " " .. guildValue .. L["Lottery Draw Step 4 Condition 1 Part 2"] .. ": " .. remainingValue .. ".", "GUILD") end)
		ms = ms + 0.2
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent == 0 and secondPercent > 0 then
		C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 Fixed Variable 2"] .. " " .. remainingValue .. ". " .. L["Lottery Draw Step 4 Fixed Variable 3"] .. ": " .. secondValue .. ".", "GUILD") end)
		ms = ms + 0.2
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent > 0 and secondPercent > 0 then
		C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 Fixed Variable 2"] .. " " .. remainingValue .. ". " .. L["Lottery Draw Step 4 Fixed Variable 3"] .. ": " .. secondValue .. ".", "GUILD") end)
		ms = ms + 0.2
	end
	
	if GuildCutStatus then
		if secondPercent ~= L["Winner Guaranteed"] and guildPercent > 0 and secondPercent > 0 then
			C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 No Guild Cut 1"], "GUILD") end)
			ms = ms + 0.25
		elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent > 0 and secondPercent == 0 then
			C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 No Guild Cut 2"], "GUILD") end)
			ms = ms + 0.25
		end
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(ms, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
		ms = ms + 0.25
		C_Timer.After(ms, function(self) SendChatMessage(L["Lottery Draw Step 4 Announce Roll"], "GUILD") end)
		ms = ms + 10
		if not Variables[14] then
			C_Timer.After(ms, LotteryDrawSearchInvalidTickets)
		else
			C_Timer.After(ms, AdvancedLotteryFindWinner)
		end
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function LotteryDrawGuildAndSecond() --Announces the percentage that the Guild and Second Place will receive.
	
	local guildPercent = tonumber(glrCharacter.Event.Lottery.GuildCut)
	local secondPercent = glrCharacter.Event.Lottery.SecondPlace
	
	if secondPercent ~= L["Winner Guaranteed"] then
		secondPercent = tonumber(secondPercent)
	end
	
	if secondPercent == L["Winner Guaranteed"] and guildPercent > 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 1"] .. guildPercent .. L["Lottery Draw Step 3 Fixed Variable 1"], "GUILD") end)
	elseif secondPercent == L["Winner Guaranteed"] and guildPercent == 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 2"], "GUILD") end)
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent > 0 and secondPercent > 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 3 Guild Part"] .. guildPercent .. L["Lottery Draw Step 3 Fixed Variable 1"], "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 3"] .. secondPercent .. L["Lottery Draw Step 3 Fixed Variable 1"], "GUILD") end)
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent == 0 and secondPercent == 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Fixed Variable 3"], "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(L["Lottery Draw Step 3 Fixed Variable 2"], "GUILD") end)
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent == 0 and secondPercent > 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Fixed Variable 3"], "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 3"] .. secondPercent .. L["Lottery Draw Step 3 Fixed Variable 1"], "GUILD") end)
	elseif secondPercent ~= L["Winner Guaranteed"] and guildPercent > 0 and secondPercent == 0 then
		C_Timer.After(0.2, function(self) SendChatMessage(L["Lottery Draw Step 3 Condition 3 Guild Part"] .. guildPercent .. L["Lottery Draw Step 3 Fixed Variable 1"], "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(L["Lottery Draw Step 3 Fixed Variable 2"], "GUILD") end)
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(12.4, BeginLotteryDrawStep4)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function LotteryDrawCountInvalidTickets() --If any Invalid Tickets exist, will announce as such.
	
	local ticketPool = {}
	local numberOfInvalidTickets = 0
	local lowestTicketNumber = glrCharacter.Event.Lottery.FirstTicketNumber
	local highestTicketNumber = glrCharacter.Event.Lottery.LastTicketBought - 1
	if glrCharacter.Event.Lottery.SecondPlace ~= L["Winner Guaranteed"] then
		highestTicketNumber = glrCharacter.Event.Lottery.LastTicketBought - 1
	end
	
	-- If Invalid Tickets exist in any way they'll be reported to Guild Chat
	if glrCharacter.Lottery.TicketPool ~= nil then
		if glrCharacter.Lottery.TicketPool[1] ~= nil then
			numberOfInvalidTickets = #glrCharacter.Lottery.TicketPool
		end
	end
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - LotteryDrawCountInvalidTickets() - Number of Invalid Tickets: " .. numberOfInvalidTickets)
	
	if numberOfInvalidTickets > 0 then
		local message_1 = string.gsub(L["Draw Step 2 Invalid Tickets"], "%%unsold", numberOfInvalidTickets)
		local message_2 = string.gsub(string.gsub(L["Draw Step 2"], "%%low", lowestTicketNumber), "%%high", highestTicketNumber)
		C_Timer.After(0.2, function(self) SendChatMessage(message_1, "GUILD") end)
		C_Timer.After(0.4, function(self) SendChatMessage(message_2, "GUILD") end)
	else
		local message = string.gsub(string.gsub(L["Draw Step 2"], "%%low", lowestTicketNumber), "%%high", highestTicketNumber)
		C_Timer.After(0.2, function(self) SendChatMessage(message, "GUILD") end)
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(12.2, LotteryDrawGuildAndSecond)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function AdvancedLotteryStep1() --Generates the required number of Tickets and assigns them at random to entered players. Also announces as such to the Guild.
	
	local first = glrCharacter.Event.Lottery.FirstTicketNumber
	local last = glrCharacter.Event.Lottery.LastTicketBought - 1
	local n = first
	local c = 0
	local c2 = 0
	local jackpotGold = tostring(tonumber(glrCharacter.Event["Lottery"]["StartingGold"]) + ( glrCharacter.Event["Lottery"]["TicketsSold"] * tonumber(glrCharacter.Event["Lottery"]["TicketPrice"]) ))
	local jackpotValue = GetValue(jackpotGold, "Lottery", "Jackpot", false)
	local numberOfTickets = glrCharacter.Event.Lottery.TicketsSold
	local message = string.gsub(string.gsub(L["Draw Step 1"], "%%TicketsSold", numberOfTickets), "%%money", jackpotValue)
	Variables[17] = glrCharacter.Event.Lottery.TicketsSold + glrCharacter.Event.Lottery.GiveAwayTickets
	
	C_Timer.After(0.2, function(self) SendChatMessage(message, "GUILD") end)
	
	-- If it's not a Guaranteed Winner Event it doubles the number of required Tickets to generate, making it a 50% Win Chance.
	if glrCharacter.Event.Lottery.SecondPlace ~= L["Winner Guaranteed"] then
		Variables[17] = Variables[17] * 2
	end
	
	-- Generates the number of Tickets, starting with the first ticket in the Event Series (ie: 1000, 2000, 3000, ...)
	-- "n" is the ticket number
	-- "c" is the total number of Tickets generated, which is reported to Guild Chat.
	for i = 1, Variables[17] do
		table.insert(EventTickets, tostring(n))
		Variables[13][2] = Variables[13][2] + 1
		n = n + 1
		c = c + 1
	end
	
	RandomizationTable.Totals = EventTickets
	
	-- Starts assigning available Ticket Numbers to players.
	for i = 1, #glrCharacter.Lottery.Entries.Names do
		-- Assigns one ticket at a time until it's assigned the number of tickets the Player has.
		for j = 1, glrCharacter.Lottery.Entries.Tickets[glrCharacter.Lottery.Entries.Names[i]].NumberOfTickets do
			-- Pulls a Random Ticket out of the available Tickets table.
			local t = math.random(1, #EventTickets)
			-- If this is the first time assigning a Ticket to a Player, creates the table holding their Ticket Numbers.
			if RandomizationTable.Names[glrCharacter.Lottery.Entries.Names[i]] == nil then
				RandomizationTable.Names[glrCharacter.Lottery.Entries.Names[i]] = {}
			end
			-- Adds the Random Ticket to the Player
			table.insert(RandomizationTable.Names[glrCharacter.Lottery.Entries.Names[i]], EventTickets[t])
			-- Assigns the Ticket Number to the Player, making it easy to search for a Winner once the Winning Ticket Number is drawn.
			RandomizationTable.Numbers[EventTickets[t]] = glrCharacter.Lottery.Entries.Names[i]
			-- Removes the Random Ticket from the table, preventing it from being drawn again.
			table.remove(EventTickets, t)
			-- Increasing number which is reported to Guild Chat, stating the number of Tickets assigned to players.
			c2 = c2 + 1
		end
	end
	
	local timestamp = GetTimeStamp()
	table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - AdvancedLotteryStep1() - Number of Tickets Required: " .. Variables[17] .. " - Generated: " .. c .. " Assigned: " .. c2)
	
	-- If Tickets were given away it'll report to Guild Chat the number Given.
	if glrCharacter.Event.Lottery.GiveAwayTickets ~= nil then
		if glrCharacter.Event.Lottery.GiveAwayTickets > 0 then
			local GivenTickets = tostring(glrCharacter.Event.Lottery.GiveAwayTickets)
			local message = L["Send Info GLR"] .. ": " .. string.gsub(L["Draw Given Tickets"], "%%t", GivenTickets)
			C_Timer.After(0.4, function(self) SendChatMessage(message, "GUILD") end)
		end
	end
	
	if not GLR_Global_Variables[3] then
		local message, _ = string.gsub(string.gsub(L["Advanced Draw Step 1"], "%%f", first), "%%l", last)
		local tickets, _ = string.gsub(string.gsub(L["Advanced Draw Step 1 Totals"], "%%t", c), "%%c", c2)
		C_Timer.After(0.6, function(self) SendChatMessage(tickets, "GUILD") end)
		C_Timer.After(0.8, function(self) SendChatMessage(message, "GUILD") end)
		C_Timer.After(12.8, LotteryDrawGuildAndSecond)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function LotteryDrawAnnounceTickets() --Announces number of Tickets Sold & Given away.
	
	local jackpotGold = tostring(tonumber(glrCharacter.Event["Lottery"]["StartingGold"]) + ( glrCharacter.Event["Lottery"]["TicketsSold"] * tonumber(glrCharacter.Event["Lottery"]["TicketPrice"]) ))
	local jackpotValue = GetValue(jackpotGold, "Lottery", "Jackpot", false)
	local numberOfTickets = glrCharacter.Event.Lottery.TicketsSold
	local message = string.gsub(string.gsub(L["Draw Step 1"], "%%TicketsSold", numberOfTickets), "%%money", jackpotValue)
	
	C_Timer.After(0.2, function(self) SendChatMessage(message, "GUILD") end)
	
	if glrCharacter.Event.Lottery.GiveAwayTickets ~= nil then
		if glrCharacter.Event.Lottery.GiveAwayTickets > 0 then
			local GivenTickets = tostring(glrCharacter.Event.Lottery.GiveAwayTickets)
			C_Timer.After(0.4, function(self) SendChatMessage(L["Send Info GLR"] .. ": " .. string.gsub(L["Draw Given Tickets"], "%%t", GivenTickets), "GUILD") end)
		end
	end
	
	if not GLR_Global_Variables[3] then
		C_Timer.After(12.2, LotteryDrawCountInvalidTickets)
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

local function InitiateLotteryRoll()
	
	Variables[13][2] = 0
	RandomizationTable = { ["Names"] = {}, ["Numbers"] = {}, ["Totals"] = {}, } --Resets table for Advanced Ticket Draws
	local lotteryStatus = glrCharacter.Data.Settings["CurrentlyActiveLottery"]
	Variables[17] = 0
	EventTickets = {}
	Variables[14] = glrCharacter.Data.Settings.AdvancedTicketDraw
	Variables[10] = false
	Variables[11] = false
	GLR_Global_Variables[3] = false
	Variables[9] = false
	Variables[13][1] = 0
	
	if not GLR_Global_Variables[3] then
		if lotteryStatus and glrCharacter.Event.Lottery ~= nil then --Prevents starting a non-existent lottery draw
			if glrCharacter.Event.Lottery.TicketsSold >= 1 or glrCharacter.Event.Lottery.GiveAwayTickets >= 1 then
				local Jackpot = tonumber(glrCharacter.Event.Lottery.StartingGold) + ( glrCharacter.Event.Lottery.TicketsSold * tonumber(glrCharacter.Event.Lottery.TicketPrice) )
				GLR_UI.GLR_MainFrame.GLR_AbortRollProcessButton:Show()
				glrLotteryDrawInProgress = true --Prevents a lottery draw from occuring while one's in progress
				local timestamp = GetTimeStamp()
				table.insert(GLR_AddOnMessageTable_Debug.Messages, timestamp .. " - " .. GLR_GameVersion .. " - InitiateLotteryRoll() - Started Event Draw; Variables[14]? " .. string.upper(tostring(Variables[14])) .. " - Tickets Sold: " .. glrCharacter.Event.Lottery.TicketsSold .. " Tickets Given Away: " .. glrCharacter.Event.Lottery.GiveAwayTickets )
				SendChatMessage("--------------------------------------------------", "GUILD")
				C_Timer.After(0.3, function(self) SendChatMessage(L["GLR Title with Version"] .. GLR_CurrentVersionString, "GUILD") end)
				if not Variables[14] then
					C_Timer.After(0.6, function(self) SendChatMessage(L["Begin Lottery Draw"], "GUILD") end)
				else
					C_Timer.After(0.6, function(self) SendChatMessage(L["Begin Advanced Lottery Draw"], "GUILD") end)
				end
				C_Timer.After(0.9, function(self) SendChatMessage("--------------------------------------------------", "GUILD") end)
				if not Variables[14] then
					C_Timer.After(15, LotteryDrawAnnounceTickets)
				else
					C_Timer.After(15, AdvancedLotteryStep1)
				end
			end
		else
			GLR:Print(L["No Lottery Active"])
		end
	elseif GLR_Global_Variables[3] and not Variables[9] then
		C_Timer.After(1, function(self) SendChatMessage(L["Abort Roll Message"], "GUILD") end)
		glrLotteryDrawInProgress = false
		Variables[9] = true
	end
	
end

--Confirm Start Raffle Roll Button Settings
local GLR_StartRaffleButton = CreateFrame("Button", "GLR_StartRaffleButton", GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_ConfirmStartRaffleBorder, "GameMenuButtonTemplate")
GLR_StartRaffleButton:SetPoint("CENTER", GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_ConfirmStartRaffleBorder, "CENTER")
GLR_StartRaffleButton:SetSize(100, 26)
GLR_StartRaffleButton:SetScript("OnClick", function(self)
	InitiateRaffleRoll()
	GLR_UI.GLR_MainFrame.GLR_RaffleFrame.GLR_ConfirmStartRaffleBorder:Hide()
end)
GLR_StartRaffleButton:SetBackdrop({
	edgeFile = profile.buttons.edgeFile,
	edgeSize = profile.buttons.edgeSize,
})
GLR_StartRaffleButton:SetBackdropBorderColor(unpack(profile.colors.buttons.bordercolor))
GLR_StartRaffleButton:SetScript("OnEnter", function(self)
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetText(L["GLR Tooltip Title"], 0.97, 1, 0.30)
	GameTooltip:AddLine(L["Confirm Start Raffle Button Tooltip"])
	GameTooltip:AddLine(L["Confirm Start Raffle Button Tooltip Warning"], 1, 0, 0)
	GameTooltip:Show()
end)
GLR_StartRaffleButton:SetScript("OnLeave", function(self)
	GameTooltip:Hide()
end)

--Confirm Start Lottery Roll Button Settings
local GLR_StartLotteryButton = CreateFrame("Button", "GLR_StartLotteryButton", GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_ConfirmStartLotteryBorder, "GameMenuButtonTemplate")
GLR_StartLotteryButton:SetPoint("CENTER", GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_ConfirmStartLotteryBorder, "CENTER")
GLR_StartLotteryButton:SetSize(100, 26)
GLR_StartLotteryButton:SetScript("OnClick", function(self)
	InitiateLotteryRoll()
	GLR_UI.GLR_MainFrame.GLR_LotteryFrame.GLR_ConfirmStartLotteryBorder:Hide()
end)
GLR_StartLotteryButton:SetBackdrop({
	edgeFile = profile.buttons.edgeFile,
	edgeSize = profile.buttons.edgeSize,
})
GLR_StartLotteryButton:SetBackdropBorderColor(unpack(profile.colors.buttons.bordercolor))
GLR_StartLotteryButton:SetScript("OnEnter", function(self)
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetText(L["GLR Tooltip Title"], 0.97, 1, 0.30)
	GameTooltip:AddLine(L["Confirm Start Lottery Button Tooltip"])
	GameTooltip:AddLine(L["Confirm Start Lottery Button Tooltip Warning"], 1, 0, 0)
	GameTooltip:Show()
end)
GLR_StartLotteryButton:SetScript("OnLeave", function(self)
	GameTooltip:Hide()
end)
--GLR_StartLotteryButton:Hide()

--Confirm Start Raffle Roll Button String & Settings
local GLR_StartRaffleButtonText = GLR_StartRaffleButton:CreateFontString("GLR_StartRaffleButtonText", "OVERLAY", "GameFontWhiteTiny")
GLR_StartRaffleButtonText:SetPoint("CENTER", GLR_StartRaffleButton)
GLR_StartRaffleButtonText:SetText(L["Confirm Start Button"])
GLR_StartRaffleButtonText:SetFont(GLR_UI.ButtonFont, 11)

--Confirm Start Lottery Roll Button String & Settings
local GLR_StartLotteryButtonText = GLR_StartLotteryButton:CreateFontString("GLR_StartLotteryButtonText", "OVERLAY", "GameFontWhiteTiny")
GLR_StartLotteryButtonText:SetPoint("CENTER", GLR_StartLotteryButton)
GLR_StartLotteryButtonText:SetText(L["Confirm Start Button"])
GLR_StartLotteryButtonText:SetFont(GLR_UI.ButtonFont, 11)

local function CheckLocale()
	locale = GetLocale()
	if locale == nil then
		C_Timer.After(1, CheckLocale)
	else
		if locale == "frFR" then
			GLR_StartLotteryButton:SetSize(135, 26)
			GLR_StartRaffleButton:SetSize(135, 26)
		end
	end
end

if locale == nil then
	CheckLocale()
else
	if locale == "frFR" then
		GLR_StartLotteryButton:SetSize(135, 26)
		GLR_StartRaffleButton:SetSize(135, 26)
	end
end